/*
 * This is part of the doxml library, a parser for XML.  It should not
 *  require any functionality not defined in ANSI C's standard libraries.
 *
 * Copyright (C) 1999 John Stracke, http://www.thibault.org
 *
 * This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public License
 *  as published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
 *
 * $Log: priv.h,v $
 * Revision 1.16  1999/07/24 20:23:51  francis
 * Adding support for param entities in the DTD
 *
 * Revision 1.15  1999/07/24 16:55:27  francis
 * Adding support for entity-valued attributes
 *
 * Revision 1.14  1999/07/24 15:23:03  francis
 * Adding support for default attribute values
 *
 * Revision 1.13  1999/07/24 14:53:14  francis
 * Adding support for attribute value normalization
 *
 * Revision 1.12  1999/07/24 13:43:46  francis
 * Adding support for namespaces in entity names.
 *
 * Revision 1.11  1999/07/19 01:31:11  francis
 * Implementing namespace reform (but not using it yet).  Not documented yet.
 *
 * Revision 1.10  1999/07/18 20:25:06  francis
 * Moving the global buff (static in parse.c) into the doxml_context,
 * to make the parser reentrant, so that it can be used from two threads
 * simultaneously.  Documenting the change; updating the version number
 * of the manual.
 *
 * Revision 1.9  1999/06/25 21:17:37  francis
 * Changing DOXML_XML to DOXML_XML; adding idents
 *
 * Revision 1.8  1999/06/25 21:15:00  francis
 * Adding a framework for the application to provide doxml with URI
 * handlers for use in processing external IDs.  (Not providing URI
 * code as part of doxml means we can work with whatever URI code the
 * app wants, and don't have to write anything non-portable.)
 *
 * Also, changing old _LIBXML_ stuff in the headers to _DOXML_.
 *
 * Revision 1.7  1999/06/25 19:43:02  francis
 * Adding support for unparsed entities
 *
 * Revision 1.6  1999/06/01 01:51:14  francis
 * Adding functions to get/set/clear attributes.  Reorganzing the
 * to-do list.  Fixing a parser bug introduced I removed support for
 * attributes with no values at release 0.2.
 *
 * Revision 1.5  1999/05/22 14:29:16  francis
 * Removing the case-insensitive compare, since it turns out XML tags are
 * case-sensitive.
 *
 * Revision 1.4  1999/05/21 23:10:24  francis
 * Reformatting, since renaming to doxml changed the widths of the
 * identifiers.
 *
 * Revision 1.3  1999/05/21 23:05:41  francis
 * Finished renaming to doxml
 *
 * Revision 1.2  1999/05/21 22:59:48  francis
 * Partially renaming to doxml
 *
 * Revision 1.1.1.1  1999/05/21 22:20:36  francis
 * Initial checkin of libxml, to be renamed doxml
 *
 */
#ifndef _DOXML_DOXML_PRIV_H_
#define _DOXML_DOXML_PRIV_H_

static const char* _doxml_version_ident_doxml_priv_h="$Id: priv.h,v 1.16 1999/07/24 20:23:51 francis Exp $";

#include <doxml/xml.h>
#include <doxml/polyuri.h>

/* An doxml_getc_method must point to a function that acts like getc():
 *  it reads from the input stream of the doxml_context pointed to by
 *  context, and returns an unsigned char cast to an int, or EOF on
 *  end of file or error.
 *
 * The method may assume that the doxml_context was constructed with
 *  closure appropriate for the method.
 *
 * The method should be capable of handling 8-bit bytes, so that the
 *  parser can be used on UTF-8 text.
 */
typedef int (*doxml_getc_method)(doxml_context* context);

/* An doxml_ungetc_method must point to a function that acts like ungetc():
 *  it takes the character and returns it to the input buffer for the
 *  next call to the doxml_getc_method.
 *
 * The method may assume that the doxml_context was constructed with
 *  closure appropriate for the method.
 *
 * The method should be capable of handling 8-bit bytes, so that the
 *  parser can be used on UTF-8 text.
 */
typedef int (*doxml_ungetc_method)(int ch,
                                   doxml_context* context);

/* A method invoked on an doxml_context prior to free()ing the doxml_context.
 *  It is legal for an doxml_context to have a destructor_method of NULL
 *  (equivalent to having a destructor_method which does nothing).
 */
typedef void (*doxml_destructor_method)(doxml_context* context);

/* Construct a doxml_context with the given methods.
 */
doxml_context* doxml_construct_context(doxml_getc_method getc_method,
                                       doxml_destructor_method destructor_method);

/* The struct that represents a parsing session.
 */
struct doxml_context_tag {
  struct {
    doxml_getc_method getc_method;
    doxml_destructor_method destructor_method;
    void* closure;
  } reader;

  int ungetc_buff;

  const doxml_markupdecl* external_entity;

  doxml_compound_uri_handler* compound_uri_handler;

  struct {
    char* data;
    int size,actual;
    char last_ch;
    int deferred_space;
  } buff;

  doxml_context* pushed;

  /* If non-NULL, then doxml_getc() will be on the lookout for %s;
   *  if it finds them, it will call doxml_doctype_handle_percent(), below.
   *  If doxml_doctype_handle_percent() fails, doxml_getc() will fail;
   *  otherwise, it will read from the newly pushed context.
   *
   * context->doctype is set when parsing a <!DOCTYPE> and cleared
   *  otherwise.
   */
  doxml_doctype* doctype;

  int line_number;
};

/* Temporarily, doxml_getc(context) will be forwarded to pushed.  When
 *  pushed reaches EOF, it will be closed.  To do this, context->pushed
 *  is set to pushed.
 *
 * This is used by the parser to support parameter entities: it uses
 *  doxml_open_string() to read from the entity definition and then
 *  pushes the resulting context.
 *
 * If context->pushed is non-NULL, doxml_context_push() will recurse to
 *  push onto context->pushed (e.g., if the literal entity value
 *  references another entity).
 */
int doxml_context_push(doxml_context* context,
                       doxml_context* pushed);

/* Reads an entity name from context and searches doctype
 *  for the corresponding param declaration.  If found, uses
 *  doxml_context_push() to start reading from the param's
 *  literal entity value.  Otherwise, returns -1.
 */
int doxml_doctype_handle_percent(doxml_context* context,
                                 doxml_doctype* doctype);

int doxml_getc(doxml_context* context);
int doxml_ungetc(int ch,
                 doxml_context* context);

/* The prefix xmlns: maps to the pseudo-namespace "" */
const char* doxml_lookup_namespace(doxml_atom* atom,
                                   const char* prefix);

const char* doxml_lookup_prefix(doxml_atom* atom,
                                const char* space);

doxml_attribute* doxml_attr_seek(doxml_atom* atom,
                                 const char* space,
                                 const char* name);
doxml_attribute* doxml_attr_seek2(doxml_atom* atom,
                                  const doxml_name_in_dtd* name);
doxml_attribute* doxml_attr_add(doxml_atom* atom,
                                const char* space,
                                const char* name,
                                const char* value);
/* If space is 0, it will be deduced from prefix */
doxml_attribute* doxml_attr_add_with_prefix(doxml_atom* atom,
                                            const char* space,
                                            const char* prefix,
                                            const char* name,
                                            const char* value);
doxml_attribute* doxml_attr_add_entity(doxml_atom* atom,
                                       const char* space,
                                       const char* name,
                                       const doxml_markupdecl* value);
/* If space is 0, it will be deduced from prefix */
doxml_attribute* doxml_attr_add_entity_with_prefix(doxml_atom* atom,
                                                   const char* space,
                                                   const char* prefix,
                                                   const char* name,
                                                   const doxml_markupdecl* value);
void doxml_attr_del(doxml_atom* element,
                    doxml_attribute* attr);

#endif /*_DOXML_DOXML_PRIV_H_*/
